# -*- coding: utf-8 -*-

import os
import re
import string
import unicodedata
import platform

from datetime import datetime
from functools import update_wrapper

from click import pass_context

from jinja2 import nodes
from jinja2.ext import Extension


def pass_tpldir(f):
    @pass_context
    def new_func(ctx, *args, **kwargs):
        if ctx.obj and "TPLDIR" in ctx.obj:
            return ctx.invoke(f, ctx.obj["TPLDIR"], *args, **kwargs)
        else:
            return ctx.invoke(f, ctx.obj["TPLDIR"], *args, **kwargs)

    return update_wrapper(new_func, f)


class JinjaTimeExtension(Extension):
    """
    Adds a {% time %} tag to jinja2

    The argument gets passed to time.strftime.
    """

    tags = {"time"}

    def __init__(self, environment):
        super(JinjaTimeExtension, self).__init__(environment)

        environment.extend(datetime_format="%Y-%m-%d")

    def _time(self, datetime_format):
        if datetime_format is None:
            datetime_format = self.environment.datetime_format

        return datetime.now().strftime(datetime_format)

    def parse(self, parser):
        lineno = next(parser.stream).lineno

        arg = parser.parse_expression()

        call_method = self.call_method(
            "_time",
            [arg],
            lineno=lineno,
        )

        return nodes.Output([call_method], lineno=lineno)


valid_filename_chars = "-_.() %s%s" % (string.ascii_letters, string.digits)
default_replace = dict(ä='ae', ö='oe', ü='ue', ß='ss')


def jinja_filter_fileify(s, sep="_", replace=default_replace):
    """
    Django util.text.get_valid_filename
    """
    # s = str(s).strip().replace(" ", "_")
    # return re.sub(r"(?u)[^-\w.]", "", s)

    # https://gist.github.com/wassname/1393c4a57cfcbf03641dbc31886123b8
    # replace spaces
    s = str(s).strip().replace(" ", sep)
    for k, v in replace.items():
        s = s.replace(k, v)

    # keep only valid ascii chars
    s = (
        unicodedata.normalize("NFKD", s).encode("ASCII", "ignore").decode()
    )

    if "windows" in platform.system().lower():
        if len(s) > 255:
            s, ext = os.path.splitext(s)
            return s[:(char_limit - len(ext))] + ext
    return s


def jinja_filter_slugify(value, allow_unicode=False):
    """
    Django util.text.slugify
    """
    value = str(value)
    if allow_unicode:
        value = unicodedata.normalize("NFKC", value)
    else:
        value = (
            unicodedata.normalize("NFKD", value)
                .encode("ascii", "ignore")
                .decode("ascii")
        )
    value = re.sub(r"[^\w\s-]", "", value.lower())
    return re.sub(r"[-\s]+", "-", value).strip("-_")


def jinja_filter_roman(value):
    """ Convert an integer to a Roman numeral.
    https://www.oreilly.com/library/view/python-cookbook/0596001673/ch03s24.html
    """

    if not isinstance(value, type(1)):
        return value
    if not 0 < value < 4000:
        return value
    ints = (1000, 900, 500, 400, 100, 90, 50, 40, 10, 9, 5, 4, 1)
    nums = (
    'M', 'CM', 'D', 'CD', 'C', 'XC', 'L', 'XL', 'X', 'IX', 'V', 'IV', 'I')
    result = []
    for i in range(len(ints)):
        count = int(value / ints[i])
        result.append(nums[i] * count)
        value -= ints[i] * count
    return ''.join(result)


def jinja_filter_time(value):
    """Pass the value to datetime.now().strftime()"""
    return datetime.now().strftime(value)



# -*- coding: utf-8 -*-

import click
from colorama import Fore, Back, Style


def printd(msg, echo=click.echo, decor=""):
    """Print msg

    Prefix with decor if given and passed to echo or returned if echo==None
    """
    if callable(echo):
        return echo(f"{decor}{msg}")
    else:
        return f"{decor}{msg}"


def info(msg, echo=click.echo):
    return printd(
        msg, echo=echo, decor=f"[{Fore.BLUE}{Style.BRIGHT}i{Style.RESET_ALL}] "
    )


def warn(msg, echo=click.echo):
    return printd(
        msg, echo=echo, decor=f"[{Fore.YELLOW}{Style.BRIGHT}!{Style.RESET_ALL}] "
    )


def error(msg, echo=click.echo):
    return printd(
        msg, echo=echo, decor=f"[{Fore.RED}{Style.BRIGHT}X{Style.RESET_ALL}] "
    )


def success(msg, echo=click.echo):
    return printd(
        msg, echo=echo, decor=f"[{Fore.GREEN}{Style.BRIGHT}✓{Style.RESET_ALL}] "
    )


def indent(msg, echo=click.echo):
    return printd(msg, echo=echo, decor="    ")


def question(msg, default=None, echo=click.prompt, color=Fore.BLUE):
    msg = printd(msg, echo=None, decor=f"[{color}{Style.BRIGHT}?{Style.RESET_ALL}] ")
    if default:
        return echo(msg, default=default)
    else:
        return echo(msg)



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['parboil']

package_data = \
{'': ['*']}

install_requires = \
['Jinja2>=3.0.2,<4.0.0', 'click>=8.0.3,<9.0.0', 'colorama>=0.4.4,<0.5.0']

entry_points = \
{'console_scripts': ['boil = parboil.parboil:boil']}

setup_kwargs = {
    'name': 'parboil',
    'version': '0.7.7',
    'description': 'Create reusable boilerplate templates to kickstart your next project.',
    'long_description': '<h1 align=center>\n:rice:\n<br>\nParboil\n</h1>\n<p align=center><strong>Project Boilerplate Generator</strong></p>\n\n![GitHub](https://img.shields.io/github/license/jneug/parboil)\n\nWith _Parboil_ you can create reusable boilerplate templates to kickstart your next project.\n<small>_Parboil_ is a Python rewrite of [boilr](https://github.com/tmrts/boilr) by [Tamer Tas](https://github.com/tmrts)</small>\n\n----\n\n## Installation\n\nInstall **Python 3** and then _Parboil_ with **pip**:\n\n```\npip install parboil\n```\n\n_Parboil_ will install a `boil` command on your system. Run `boil --version` to see, if it worked.\n\n## Getting started\n\nUse `boil --help` to see the list of available commands and `boil <command> --help` to see usage information for any command.\n\n### Installing your first template\n\n_Parboil_ maintains a local repository of project templates. To use _Parboil_ you first need to install a template. You can install templates from a local directory or download them from GitHub.\n\nFor your first template install `jneug/parboil-template` from GitHub - a project template to create parboil project templates.\n\n```\nboil install -d jneug/parboil-template pbt\n```\n\nThis will download the template from [`jneug/parboil-template`](https://github.com/jneug/parboil-template) and makes it available under the name `pbt`. (The `-d` flag tells Parboil, that you want to download from GitHub and not from a local directory.)\n\nVerify the install with `boil list`.\n\n### Using a template\n\nTo use your new template run\n\n```\nboil use pbt new_template\n```\n\nThis will create the boilerplate project in the `new_template` directory. (Omitting the directory will add the template to the current working dir.) _Parboil_ asks you to input some data and then writes the project files.\n\n### Uninstall and update\n\nTo remove a template run `boil uninstall <templatename>` and to update from its original source (either a local directory or a GitHub repository) run `boil update <templatename>`. \n\n### Creating your first template\n\nThe parboil-template is a good startign point to create your own template. _Parboil_ uses [Jinja2](https://jinja.palletsprojects.com) to parse the template files and dynamically insert the user information into the template files. That means, you can use all of Jinjas features (and some more), to create your template files. \n\nLet\'s create a simple project template for meeting logs from scratch.\n\nFirst, create a directory for your new template. Let\'s call it `meeting_log`. Then creat a directory called `template` and a file called `project.json` in it.\n\n```\nmeeting_log\n\ttemplate\n\tproject.json\n```\n\nThis is the basic structure of a project template. `project.json` holds the template configuration and `template` the actual template files.\n\nNow open `project.json` in any editor and copy the following text into it:\n\n```\n{\n\t"fields": {\n\t\t"Author": "",\n\t\t"Meeting": "Daily meeting",\n\t\t"MeetingNo": 1,\n\t\t"Topic": "Planning the day",\n\t\t"IamModerator": false\n\t}\n}\n```\n\nThe `fields` key is used to define custom inputs, that the user needs to enter whenever the template is used. The key of each entry is the field name, the value is the default. An empty string means, the key is required everytime.\n\nNow we need to add the file(s) that should be created by Parboil. Create `{{Meeting}}_log.md` in the `template` directory. And enter this text:\n\n```\n# Meeting notes for {{ Meeting|title }} #{{ MeetingNo }} \n\n> Date: {% time \'%Y-%m-%d\' %}\n> Topic: {{ Topic }}\n> Author: {{ Author }}\n{% if IamModerator %}> Moderator: {{ Author }}{% endif %}\n\n## Topics\n\n1. \n2. \n\n## Notes\n\n\n```\n\nThe template uses [Jinjas syntax](https://jinja.palletsprojects.com/en/2.11.x/templates/) to add the field values. For example `{{ Author }}` will be replaced with the name you entered while prompted. Note that you can use the fields in filenames, too.\n\nYou can use any Jinja [macros](https://jinja.palletsprojects.com/en/2.11.x/templates/#list-of-control-structures) and [filters](https://jinja.palletsprojects.com/en/2.11.x/templates/#list-of-builtin-filters) in your templates. `{{ Meeting|title }}` will tranform the value of "Meeting" into titlecase. `{% if IamModerator %}` is a conditional. \n\nFor more information read [the wiki page on template creation](https://github.com/jneug/parboil/wiki/How-to-create-templates).\n\n### Some more template creation\n\nYou can do some more complex stuff with templates. For example you might want to name the logfile in the example above with the current date and the meetings number padded with zeros to two digits. Also, the meeting name should be filtered for use in filenames. You would need to name the file like this:\n\n```\n{% time \'%Y-%m-%d\' }_{{ \'{:02}\'.format(MeetingNo) }}-{{\xa0Meeting|fileify }}.md\n```\n\nThe use of special chars works on many systems, but now all. Also, the filename becomes hard to read.\n\nTo deal with this, you can rename your files from the `project.json` config file. Add a `files` object next to the `fields` and map the filenames to the rename pattern:\n\n\n```\n{\n\t"fields": {\n\t\t...\n\t},\n\t"files": {\n\t\t"meeting-log.md": "{% time \'%Y-%m-%d\' }_{{ \'{:02}\'.format(MeetingNo) }}-{{\xa0Meeting|fileify }}.md"\n\t}\n}\n```\n\nNow you can name the log file `meeting-log.md` and will get something like `2021-03-10_04-Daily Standup.md` as a result.\n\nThere are some more features for creating complex templates, like subtemplates (for example run a template to generate a license file inside differnt app project templates), selective file inclusion, template inheritance or dealing with empty files.\n',
    'author': 'J. Neugebauer',
    'author_email': 'github@neugebauer.cc',
    'maintainer': None,
    'maintainer_email': None,
    'url': 'http://github.com/jneug/parboil',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'entry_points': entry_points,
    'python_requires': '>=3.6.2,<4.0.0',
}


setup(**setup_kwargs)
